<?php

/**
 * Defines the form to be shown on the Geo-comments admin page at
 * admin/settings/geo_comments.
 */
function geo_comments_admin_form($state) {
  $form = array();

  $form['msg'] = array(
    '#type' => 'item',
    '#value' => t('Here you can configure the Geo-Comments module which extends the Comment form with a Google map, allowing visitors to "tag" their location in the world when they submit a comment.'),
  );

  // The Google Map itself.
  $form['map'] = array(
    '#type' => 'markup',
    '#theme' => 'geo_comments_admin_map',
  );
  $form['enable_map_constraints'] = array(
    '#type' => 'checkbox',
    '#id' => 'toggleMapConstraints',
    '#default_value' => variable_get('geo_comment_enable_map_constraints', false),
    '#title' => t('Enable Map Bounds'),
    '#description' => t('Designate a rectangular region on the map. Panning will be restricted to this zone and marker pins can only be placed within its boundary.'),
  );

  // Global default settings for the enable/require location tagging toggles.
  $form['geo'] = array(
    '#type' => 'fieldset',
    '#tree' => TRUE,
    '#collapsible' => FALSE,
    '#title' => t('Geo-comments Node Defaults'),
    '#description' => t('Global default toggles for enabling/requiring Geo-comments on all nodes.'),
  );
  $form['geo']['geo_comment_default_mode'] = array(
    '#type' => 'select',
    '#title' => t('Default location tagging mode'),
    '#default_value' => variable_get('geo_comment_default_mode', 'enabled'),
    '#options' => array(
      'disabled' => 'Disabled',
      'enabled' => 'Enabled',
      'required' => 'Required',
    ),
    '#description' => t('Set the default mode for Geo-comments to work with nodes on your site. This setting can be overridden on a per-node basis.'),
    '#id' => 'locationTaggingMode',
  );
  // Default map type.
  $form['geo']['geo_comment_map_type_control'] = array(
    '#type' => 'select',
    '#title' => t('Default map type'),
    '#default_value' => variable_get('geo_comment_map_type_control', 'road'),
    '#options' => array(
      'road' => 'Road Map',
      'satellite' => 'Satellite Map',
    ),
    '#description' => t('Choose between the road map view and the satellite map view.'),
    '#id' => 'mapTypeControl',
  );

  // Default map center location.
  $default_map_center = array(
    'lat' => variable_get('geo_comment_default_map_center_lat', 0),
    'lng' => variable_get('geo_comment_default_map_center_lng', 0),
  );
  $form['geo_comment_default_map_center_lat'] = array(
    '#type' => 'hidden',
    '#default_value' => $default_map_center['lat'],
    '#id' => 'mapCenterLat',
  );
  $form['geo_comment_default_map_center_lng'] = array(
    '#type' => 'hidden',
    '#default_value' => $default_map_center['lng'],
    '#id' => 'mapCenterLng',
  );

  // Map panning bounds.
  $map_pan_constraints = array(
    'geo_comment_map_min_lat' => variable_get('geo_comment_map_min_lat', ''),
    'geo_comment_map_max_lat' => variable_get('geo_comment_map_max_lat', ''),
    'geo_comment_map_min_lng' => variable_get('geo_comment_map_min_lng', ''),
    'geo_comment_map_max_lng' => variable_get('geo_comment_map_max_lng', ''),
  );
  $form['geo_comment_map_min_lat'] = array(
    '#type' => 'hidden',
    '#default_value' => $map_pan_constraints['geo_comment_map_min_lat'],
    '#id' => 'mapMinLat',
  );
  $form['geo_comment_map_max_lat'] = array(
    '#type' => 'hidden',
    '#default_value' => $map_pan_constraints['geo_comment_map_max_lat'],
    '#id' => 'mapMaxLat',
  );
  $form['geo_comment_map_min_lng'] = array(
    '#type' => 'hidden',
    '#default_value' => $map_pan_constraints['geo_comment_map_min_lng'],
    '#id' => 'mapMinLng',
  );
  $form['geo_comment_map_max_lng'] = array(
    '#type' => 'hidden',
    '#default_value' => $map_pan_constraints['geo_comment_map_max_lng'],
    '#id' => 'mapMaxLng',
  );

  // Map zoom limits.
  $map_zoom_constraints = array(
    'default' => variable_get('geo_comment_default_map_zoom', 1),
    'min' => variable_get('geo_comment_min_map_zoom', 'none'),
    'max' => variable_get('geo_comment_max_map_zoom', 'none'),
  );
  $form['zoom'] = array(
    '#type' => 'fieldset',
    '#tree' => TRUE,
    '#title' => t('Map Zoom'),
    '#collapsible' => FALSE,
    '#description' => t('Specify initial and minimum/maximum map zoom levels.'),
  );
  $form['zoom']['geo_comment_default_map_zoom'] = array(
    '#type' => 'select',
    '#title' => t('Default map zoom level'),
    '#default_value' => $map_zoom_constraints['default'],
    '#options' => range(0, 20),
    '#description' => t('Suggested values: 2 for Asia; 3 for North America or South America; 4 for USA, Europe or China; 5-6 for UK, France or Germany'),
    '#element_validate' => array('geo_comment_default_map_zoom_validate'),
    '#id' => 'mapZoomDefault',
  );
  $form['zoom']['geo_comment_min_map_zoom'] = array(
    '#type' => 'select',
    '#title' => t('Minimum map zoom level'),
    '#default_value' => $map_zoom_constraints['min'],
    '#options' => array_merge(array('none' => 'None'), range(0, 20)),
    '#description' => t('Furthest zoom out; lower numbers are further away'),
    '#id' => 'mapZoomMin',
    '#element_validate' => array('geo_comment_min_map_zoom_validate'),
  );
  $form['zoom']['geo_comment_max_map_zoom'] = array(
    '#type' => 'select',
    '#title' => t('Maximum map zoom level'),
    '#default_value' => $map_zoom_constraints['max'],
    '#options' => array_merge(array('none' => 'None'), range(0, 20)),
    '#description' => t('Closest zoom in; higher numbers are closer'),
    '#id' => 'mapZoomMax',
  );

  // Map location search settings.
  $address_search_suffix = variable_get('geo_comment_address_search_suffix', '');
  $form['search'] = array(
    '#type' => 'fieldset',
    '#tree' => TRUE,
    '#collapsible' => FALSE,
    '#title' => t('Location Search Settings'),
    '#description' => t('Settings relating to the location search feature of the Google Map.'),
  );
  $form['search']['geo_comment_address_search_suffix'] = array(
    '#type' => 'textfield',
    '#title' => t('Location search suffix'),
    '#default_value' => $address_search_suffix,
    '#description' => t('(Optional) Suffix to append to location search queries. This will help to target location searches to a specific region, such as "uk" or "europe".'),
  );

  // Form submission.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}


/**
 * Form handling for geo_comments_admin_form().
 */
function geo_comments_admin_form_submit($form, $state) {
  // Save the default states of the location tagging mode - disabled, enabled or required.
  variable_set('geo_comment_default_mode', $state['values']['geo']['geo_comment_default_mode']);

  // Save the default map type value.
  variable_set('geo_comment_map_type_control', $state['values']['geo']['geo_comment_map_type_control']);

  // Save the default map center (and hence initial Geo-comment marker) position.
  variable_set('geo_comment_default_map_center_lat', $state['values']['geo_comment_default_map_center_lat']);
  variable_set('geo_comment_default_map_center_lng', $state['values']['geo_comment_default_map_center_lng']);

  // Save the minimum and maximum latitude and longitude bounds for the map.
  variable_set('geo_comment_enable_map_constraints', $state['values']['enable_map_constraints']);
  variable_set('geo_comment_map_min_lat', $state['values']['geo_comment_map_min_lat']);
  variable_set('geo_comment_map_max_lat', $state['values']['geo_comment_map_max_lat']);
  variable_set('geo_comment_map_min_lng', $state['values']['geo_comment_map_min_lng']);
  variable_set('geo_comment_map_max_lng', $state['values']['geo_comment_map_max_lng']);

  // Save the default, minimum and maximum map zoom level constraints.
  variable_set('geo_comment_default_map_zoom', $state['values']['zoom']['geo_comment_default_map_zoom']);
  variable_set('geo_comment_min_map_zoom', $state['values']['zoom']['geo_comment_min_map_zoom']);
  variable_set('geo_comment_max_map_zoom', $state['values']['zoom']['geo_comment_max_map_zoom']);

  // Save the optional search suffix for the location search box.
  variable_set('geo_comment_address_search_suffix', $state['values']['search']['geo_comment_address_search_suffix']);
}


/**
 * Validates the default zoom field.
 */
function geo_comment_default_map_zoom_validate($element, &$state) {
  if (empty($element['#value'])) {
    form_error($element, t('This field is required.'));
  }
  if ($state['values']['zoom']['geo_comment_min_map_zoom'] != 'none' && $element['#value'] < $state['values']['zoom']['geo_comment_min_map_zoom']) {
    form_error($element, t('The default zoom level must be at least the minimum zoom level.'));
  }
  if ($state['values']['zoom']['geo_comment_max_map_zoom'] != 'none' && $element['#value'] > $state['values']['zoom']['geo_comment_max_map_zoom']) {
    form_error($element, t('The default zoom level must be at most the maximum zoom level.'));
  }
}


/**
 * Validates the minimum and maximum map zoom fields. Note that we only need to
 * call this validation function on one of the two form fields because by its
 * nature it must check them both.
 */
function geo_comment_min_map_zoom_validate($element, &$state) {
  if (empty($element['#value'])) {
    form_error($element, t('This field is required.'));
  }
  if ($element['#value'] != 'none' && $state['values']['zoom']['geo_comment_max_map_zoom'] != 'none' && $element['#value'] > $state['values']['zoom']['geo_comment_max_map_zoom']) {
    form_error($element, t('The minimum zoom level cannot be higher than the maximum zoom level.'));
  }
}
